/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * Project  Zero
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
var debugEnabled=true;

/**
 * Print a message to the debug console. 
**/
function debug(message){
	console.debug(document.location.host + " : " + message);
}


var smash = {};

// Ideally, should use a closure for private (and public) data and functions,
// but this was easier for the initial SMash refactoring.

// the private values in the smash provider object
smash._providerName = "http://providers.openajax.org/smash";
smash._providerVersion = "0.1";

smash._singletonManager = undefined; // the singleton that implements all the manager-side SPI
smash._singletonClientHub = undefined; // the singleton that implements all the client-side SPI
smash._oahubspi = undefined; // private copy of OpenAjax.hubspi. Defined when successfully register with OpenAjax

smash._goodbyeMessage = undefined; // The goodbye message sent when unloading the mashup page. Protects against malicious unloading of the mashup application. If undefined, no message is displayed
smash._loadTimeout = 20000; // The default timeout time during loading of a component. The lower the value the higher the security against frame-phishing but also the higer the chance of false detections.
smash._initialClientURI = undefined; // For use by the smash provider loaded by a client. Should only be changed from the default value if the client does URI redirection at load time. Otherwise, we will assume that the current URI was also the initial URI

// --- security token stuff ---
// configurable pseudo random number generator (prng) to use for generating the security token. 
// If not set, we use Math.random. 
// If set, the provided random number generator must support a function random(strlength:integer)
// that returns a string of length strlength, where each character is a "modified Base64 for URL" character.
// This includes A-Z, a-z, and 0-9 for the first 62 digits, like standard Base64 encoding, but
// no padding '='. And the '+', '/' characters of standard Base64 are replaced by '-', '_'.
smash._prng = undefined; 
smash._securityTokenLength = 6; // configurable security token length. If default value is not used, both manager and clients have to change it to the same value. 
smash._securityTokenOverhead = null; // the number of characters in a serialized message consumed by the security tokens
smash._computeOtherTokenConstants = function() {
	smash._securityTokenOverhead = 2*smash._securityTokenLength;
	smash._multiplier = Math.pow(10, smash._securityTokenLength-1);
}
smash._computeOtherTokenConstants();

/**
 * Randomly generates the security token which will be used to ensure message integrity.
 */
smash._keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
smash._generateSecurityToken = function() {
	var r;
	if (smash._prng) 
		r = smash._prng.random(smash._securityTokenLength);
	else {
		var r1 = Math.random(); // value in (0,1)
		r = "";
		// assuming one Math.random() value has enough bits for smash._securityTokenLenght
		for (var i=0; i<smash._securityTokenLength; i++) {
			var r2 = r1 * 64; // get the most significant base-64 value
			var c = Math.floor(r2);
			r1 = (r2 - c); // the remaining fractional value
			r = r + smash._keyStr.charAt(c);
		}
	}
	return r;
}
//----------------------------- SPI ------------------------------------------------------------
smash.getName = function() { return smash._providerName; };
smash.getVersion = function() { return smash._providerVersion; };

/** 
 * boolean smash.configure(providerConfig:object) 
 */
smash.configure = function(providerConfig) {
	if (providerConfig != null) {
		// configurable tunnelURI
		if (providerConfig.tunnelURI != null) {
			smash.SEComm.tunnelURI = providerConfig.tunnelURI;
		}
		// configurable goodbyeMessage: protects against malicious unloading of the mashup application
		if (providerConfig.goodbyeMessage != null) {
			smash._goodbyeMessage = providerConfig.goodbyeMessage;
		}
		// configurable loadTimeout for clients
		if (providerConfig.loadTimeout != null) {
			smash._loadTimeout = providerConfig.loadTimeout;
		}
		// configurable securityTokenLength
		if (providerConfig.securityTokenLength != null) {
			smash._securityTokenLength = providerConfig.securityTokenLength;
			smash._computeOtherTokenConstants();
		}
		// configurable PRNG
		if (providerConfig.prng != null) { 
			smash._prng = providerConfig.prng;
		}
		// configurable initialClientURI: only for those clients which perform URI redirection
		// at client load time
		if (providerConfig.initialClientURI) {
			smash._initialClientURI = providerConfig.initialClientURI;
		}
	}
};

//---------- manager-side SPI ------------------------------------------------------------------
/**
 * boolean smash.isConnected(clientName:string)
 */
smash.isConnected = function(clientName) {
	smash._ensureSingletonManager();
	return smash._singletonManager.isConnected(clientName);
}

/**
 * smash.sendEvent(clientName:string, topic: string, data:JSON|string, matchingSubs:array of string)
 */
smash.sendEvent = function(clientName, topic, data, matchingSubs) {
	smash._ensureSingletonManager();
	smash._singletonManager.sendEvent(clientName, topic, data, matchingSubs);
}

//---------- manager-side gadget load helper ---------------------------------------------------
/**
 * Prepares for loading of a client in a separate iframe. In addition to setting up internal data-structures,
 * it updates the URI (potentially adding a fragment identifier and URI parameters). 
 * The updates are necessary to pass values needed to bootstrap communication.
 *
 * string smash.prepareForLoad({clientName: string, uri: string, 
 *  [commErrorCallback:function(clientName:string, error:string)]})
 * return value of null indicates failure, a non-null return value is the updated URI
 */
smash.prepareForLoad = function(params) {
	smash._ensureSingletonManager();
	return smash._singletonManager.prepareForLoad(params);
}

/**
 * Cleans up data-strucrures for communication with the given client. Needs to be called prior to unloading of the
 * client to prevent false positives about 'frame phishing' attacks.
 * smash.prepareForUnload(clientName: string)
 */
smash.prepareForUnload = function(clientName) {
	smash._ensureSingletonManager();
	return smash._singletonManager.prepareForUnload(clientName);
}
//------------------------- manager-side implementation -------------------------------------------
/** 
 * lazy creation of the manager-side singleton
 */
smash._ensureSingletonManager = function() {
	if (smash._singletonManager == null)
		smash._singletonManager = new smash.SEHub();
}

/**
 * Constructor.
 * The name SEHub is legacy. The provider on the manager-side does not implement any of the hub functionality
 * other than communication.
 */
smash.SEHub = function(){
	// This is used to make the object available to the private methods. This is a workaround for an error in the ECMAScript Language Specification which causes this to be set incorrectly for inner functions. See http://www.crockford.com/javascript/private.html
	var that=this;
	// associative array indexed by componentId. Each element is a ComponentInfo object. 
	// Component is synonymous with client. componentId is the same as clientName
	this.componentInfo = [];
	/**
	 * Constructor for ComponentInfo objects
	 */
	function ComponentInfo(uri, eCallback) {
		this.uri = uri;
		this.state = smash.SEHubConstants.START;
		this.errorCallback = eCallback;
	}
	// securityListener function registered for each component's security events
	function securityListener(errorType, clientName) {
		var errorString = that.getSecurityErrorString(errorType); // get the error as a string
		var ci = that.componentInfo[clientName];
		if (ci != null) {
			var errorCallback = ci.errorCallback; // the errorCallback registered by the application
			if (errorCallback != null) { // if one was registered
				errorCallback(clientName, errorString);
			}
		}
	}


	/** 
	 * string prepareForLoad({clientName: string, uri: string, 
	 *  [commErrorCallback:function(clientName:string, error:string)]})
	 * return value of null indicates failure, a non-null return value is the updated URI
	 */
	this.prepareForLoad = function(params) {
		var clientName = params.clientName; // componentId and clientName are the same thing in this code
		var componentURI = params.uri;
		if ((clientName == null) || (componentURI == null))
			return null;
		if (that.componentInfo[clientName] != null) {
			return null;
		}
		that.componentInfo[clientName] = new ComponentInfo(componentURI, params.commErrorCallback);
		that.componentInfo[clientName].seComm = new smash.SEComm(); //The SEComm library used for this component
		that.componentInfo[clientName].seComm.setSecurityListener(securityListener);
		return that.componentInfo[clientName].seComm.prepareForLoad(clientName, componentURI, that, smash._loadTimeout);
	}

	/**
	 * boolean prepareForUnload(clientName: string)
	 */
	this.prepareForUnload = function(clientName) {
		if (!that.componentInfo[clientName]) {
			// component does not exist.
			return true;
		}
		// change state. pretty useless, since going to delete anyway
		that.componentInfo[clientName].state = smash.SEHubConstants.UNLOADED;
		that.componentInfo[clientName].seComm.prepareForUnload();
		// remove the relevant objects
		that.componentInfo[clientName] = undefined;
		return true;
	}

	/**
	 * boolean isConnected(clientName:string)
	 */
	this.isConnected = function(clientName) {
		if (that.componentInfo[clientName])
			if (that.componentInfo[clientName].state == smash.SEHubConstants.LOADED)
				return true;
		return false;
	}

	/** 
	 * sendEvent(clientName:string, topic: string, data:JSON|string, matchingSubs:array of string)
	 */
	this.sendEvent = function(clientName, topic, data, matchingSubs) {
		// send to the component
		if (that.isConnected(clientName)) {
			var comms = that.componentInfo[clientName].seComm;
			if (comms) {
				comms.distribute(topic, matchingSubs, data);
			}
		}
	}

	/** Callback when component loaded */
	this.componentLoaded = function(clientName) {
		if (that.componentInfo[clientName]) {
			that.componentInfo[clientName].state = smash.SEHubConstants.LOADED;
			if (smash._oahubspi != null)
				smash._oahubspi.registerClient(smash, clientName)
		}
	}

	
	/**
	 * A message received from a component
	 * @param componentId The component that sent the message
	 * @param topic 
	 * @param message The payload of the message (JSON|string)
	 */
	this.publishInternal = function(componentId, topic, message) {
		if (that.componentInfo[componentId]) {
			// component exists
			if (smash._oahubspi != null)
				smash._oahubspi.publishEvent(componentId, topic, message)
		}
	}

	/**
	 * A subscribe message received from a component
	 * @param componentId The component that sent the message
	 * @param subId The subscription id
	 * @param topic 
	 * @return true if subscribe was accepted else false
	 */
	this.subscribeInternal = function(componentId, subId, topic) {
		if (smash._oahubspi != null) {
			return smash._oahubspi.subscribeReq(componentId, topic, subId)
		}
		return false;
	}

	/**
	 * A unsubscribe message received from a component
	 * @param componentId The component that sent the message
	 * @param subId
	 * @return true if unsubscribe was accepted else false
	 */
	this.unsubscribeInternal = function(componentId, subId) {
		if (smash._oahubspi != null) {
			return smash._oahubspi.unsubscribeReq(componentId, subId);
		}
		return false;
	}

	/**
	 * Converts a security error code into a readable error message.
	 * @param error The error code.
	 */
	this.getSecurityErrorString = function(error) {
		switch (error) {
			case smash.SecurityErrors.INVALID_TOKEN: return smash.SecurityErrors.INVALID_TOKEN_MSG;
			case smash.SecurityErrors.TOKEN_VERIFICATION_FAILED: return smash.SecurityErrors.TOKEN_VERIFICATION_FAILED_MSG;
			case smash.SecurityErrors.TUNNEL_UNLOAD: return smash.SecurityErrors.TUNNEL_UNLOAD_MSG;
			case smash.SecurityErrors.COMPONENT_LOAD: return smash.SecurityErrors.COMPONENT_LOAD_MSG;
			default: return "UNKNOWN";
		}
	}
	
	
	/**
	 * Sets the unload function which shows the goodbye message.
	 */	
	window.onunload=function(){
		if (smash._goodbyeMessage != undefined)
			alert(smasg._goodbyeMessage);
	}
}

//---------- client-side SPI -------------------------------------------------------------------

/**
 * smash.connect({[host:string], clientName:string, callback:function, [connectionConfig:object]})
 * return value is a connHandle
 */
smash.connect = function(params) {
	if ((params.host != null) || (params.callback == null)) {
		return null;
	}
	// allow a null clientName since the SMash provider can find it in the fragment.
	if (smash._singletonClientHub == null) {
		smash._singletonClientHub = new smash.SEHubClient(params.clientName, params.callback);
	}
	// function(success:boolean, connHandle:object)
	params.callback(true, smash._singletonClientHub);
	return smash._singletonClientHub;
}

/**
 * SEHubClient implementation linking the SECommClient together with the component side logic.
 */
smash.SEHubClient = function(clientName, connectCallback){
	//-------- interface implemented by connHandle in Hub 1.1. We use the SEHub instance itself
	//-------- as the connHandle object for the "manager".

	this.equals = function(anotherConn) { return that === anotherConn; }
	this.isConnected = function() { return true; }
	this.getClientName = function() { return clientName; }
	this.disconnect = function(callback) {
		callback(true, that);
		return;
	}
	/**
	 * connHandle.subscribe(topic:string, callback:function, eventCallback:function)
	 * returns a subHandle object, or null if it fails immediately.
	 */
	this.subscribe = function(topic, callback, eventCallback) {
		// keep track of the callback so that the incomming message can be distributed correctly
		var subId = (subCount + ''); // assign the subscription id - making it a string
		subCount++;
		subHandles[subId] = new SubHandle(subId, topic, callback, eventCallback, that);
		seCommClient.subscribe(subId, topic);
		return subHandles[subId];
	}
	/**
	 * connHandle.publish(topic:string, data:JSON|string)
	 */
	this.publish = function(topic, data) {
		seCommClient.publish(topic,data);
		return true;
	}
	function SubHandle(subId, topic, callback, eventCallback, sehubClient) {
		var _isSubscribed = false;
		var that = this;
		this.getTopic = function() {
			return topic;
		}
		this.getConnHandle = function() {
			return sehubClient;
		}
		this.equals = function(anotherSubHandle) {
			if ((anotherSubHandle._getSubId != null) && (typeof anotherSubHandle._getSubId == "function")
				&& (anotherSubHandle.getConnHandle != null) && (typeof anotherSubHandle.getConnHandle == "function")) {
					if ((subId === anotherSubHandle._getSubId()) && (sehubClient === anotherSubHandle.getConnHandle()))
						return true;
				}
			return false;
		}
		this.isSubscribed = function() {
			return _isSubscribed;
		}
		this.unsubscribe = function(callback) {
			return sehubClient._unsubscribe(that, callback);
		}
		this._getSubId = function() {
			return subId;
		}
		this._setIsSubscribed = function(value) {
			_isSubscribed = value;
		}
		this._getCallback = function() {
			return callback;
		}
		this._getEventCallback = function() {
			return eventCallback;
		}
	}
	//-------- end of interface implemented by connHandle in Hub 1.1.

	//------- addition public interfaces not part of Hub 1.1 -----
	/**
	 * Set a callback to find out about security errors.
	 * Not part of the OpenAjax Hub 1.1 standard
	 */
	this.setSecurityErrorCallback = function(errorcallback) {
		securityErrorCallback = errorcallback;
	}
	this.getManagerDomain = function() { 
		if (seCommClient != null && seCommClient.parsedTunnelUrl != null) 
			return seCommClient.parsedTunnelUrl.host;
	}
	
	//------- private stuff ------
	/**
	 * _unsubscribe(subHandle:object, callback:function)
	 * returns a subHandle object, or null if it fails immediately.
	 */
	this._unsubscribe = function(subHandle, callback) {
		var subId = subHandle._getSubId();
		subHandles[subId] = undefined;
		seCommClient.unsubscribe(subId);
		// no async callback as no confirmation message from manager
		if (callback != null) {
			callback(true, subHandle); // function(success:boolean, subHandle:object).
		}
		return subHandle;
	}
	var securityErrorCallback = undefined; // securityErrorCallback registered by the application in this component/frame
	// subscriptions: each subscription is assigned an integer id that is unique to this client
	var subCount = 0;
	// mapping the subscription ids to the SubHandles
	var subHandles=new Array();
	// SECommClient serving the communication between the SEHub and the SEHub client
	var seCommClient=new smash.SECommClient(clientName);
	var state = smash.SEHubConstants.LOADED; // initialize my state to LOADED.
	// This is used to make the object available to the private methods. This is a workaround for an error in the ECMAScript Language Specification which causes this to be set incorrectly for inner functions. See http://www.crockford.com/javascript/private.html
	var that=this;

	/**
	 * Processes messages received by the SECommClient
	 * @param message The actual message.
	 */
	function handleIncomingMessage(message){
		switch (message.type) {
		case smash.SECommMessage.DISTRIBUTE:
			if ((message.additionalHeader != null) && (message.additionalHeader.s != null)) {
				var subs = message.additionalHeader.s;
				for (var i=0; i < subs.length; i++) {
					var subId = subs[i];
					if ((subId != null) && (subHandles[subId] != null)) {
						var eventCallback = subHandles[subId]._getEventCallback();
						if (eventCallback != null)
							eventCallback(subHandles[subId], message.topic, message.payload);
					}
				}
			}
			break;
		case smash.SECommMessage.SUBSCRIBE_ACK:
			if (message.additionalHeader != null) {
				var subId = message.additionalHeader.subId;
				var isOk =  message.additionalHeader.isOk;
				var err = message.additionalHeader.err;
				if ((subId != null) && (isOk != null)) {
					if (subHandles[subId] != null) {
						var callback = subHandles[subId]._getCallback();
						if (isOk) {
							subHandles[subId]._setIsSubscribed(true);
							if (callback != null)
								callback(true, subHandles[subId]);
						}
						else {
							if (callback != null)
								callback(false, subHandles[subId]);
							subHandles[subId] = undefined; // unsubscribe
						}
					}
				}
			}
			// else ignore the message
			break;
		}
	}
	function securityListenerCallback(errorcode) {
		var errorString = getSecurityErrorString(errorcode);
		if (securityErrorCallback != null) {
			securityErrorCallback(errorString);
		}
		else {
			throw new Error(errorString);
		}
	}
	function getSecurityErrorString(error) {
		switch (error) {
			case smash.SecurityErrors.INVALID_TOKEN: return smash.SecurityErrors.INVALID_TOKEN_MSG;
			default: return "UNKNOWN";
		}
	}

	// Override the SECommClient's received method with our own implementation
	seCommClient.handleIncomingMessage=handleIncomingMessage;
	seCommClient.setSecurityListener(securityListenerCallback);
}
//-----------------------------------------------------------------------------------------------
smash.SEHubConstants = {

	// Constants representing states of a component.
	// Component State Machine: START -> LOADED -> UNLOADED

	START: 0,
	LOADED: 1,
	UNLOADED: 2,

};
//-----------------------------------------------------------------------------------------------
/**
 * Constants representing the different types of attacks that can be detected and prevented by the library.
 */
smash.SecurityErrors = {


	// This error occurs when the CommLib detects a message with a different security token than the one with wich it was initialized.
	INVALID_TOKEN: 0,
	INVALID_TOKEN_MSG: "The sender of the received message could not be verified because the received security token was not correct.",
	// This error occurs when the SEComm receives a different security token than the one that was sent by the SEComm during the loading of the component.
	TOKEN_VERIFICATION_FAILED: 1,		
	TOKEN_VERIFICATION_FAILED_MSG: "The security token could not be verified. A different security token than the one that was sent during the loading of the component was received after loading.",
	// Phishing error
	TUNNEL_UNLOAD: 2,
	TUNNEL_UNLOAD_MSG: "The tunnel was unloaded without the component being unloaded by the mashup application. Frame-phishing may have occured after the component was loaded successfully.",
	// Phishing error before successfull load
	COMPONENT_LOAD: 3,
	COMPONENT_LOAD_MSG: "A timeout occured before the communication channel between the component and the mashup application was set up correctly. Frame-phishing may have occured during the loading of the component."
};
//-----------------------------------------------------------------------------------------------
/**
 * The object implementing the message serializer and deserializer for use in SEComm.
 * The topic and payload are typically under application control and may contain URI reserved characters.
 * These will be percent-encoded and decoded, and the application has to deal with the composition issues
 * if it is passing in data or topics that are already percent-encoded. 
 */


smash.SECommMessage = function(){
	// The type of the message. A string
	this.type=null;
	// The topic of the message. A string
	this.topic=null;
	// The remaining header information. A JSON object
	this.additionalHeader=null;
	// The payload of the message. A string
	this.payload=null;
	// The name used in the name value pair transmission. one character for efficiency. only use a letter or number
	var typeName="y";
	var topicName="t";
	var additionalHeaderName = "h"; // other header information that is not handled by typeName and topicName
	var payloadName="p";

	/**
	 * Serializes the message into a string which can be transmitted over a communication channel.
	 * URI-encodes the topic and payload and uses "=", "&" as separators. The communication channel
	 * must not perform any URI-encoding as "=", "&" are not reserved for fragments. 
	 * If using something other than fragment messaging at the communication channel, the serialization
	 * may need to change.
	 * @return The serialized message.
	 */
	this.serialize=function(){
		var returnValue = typeName + "=" + this.type;
		if (this.topic != null) {
			var topicString = encodeURIComponent(this.topic);
			var topicSer = "&" + topicName + "=" + topicString;  
			returnValue += topicSer;
		}
		if (this.additionalHeader != null) {
			var headerString = encodeURIComponent(JSON.stringify(this.additionalHeader));
			var headerSer = "&" + additionalHeaderName + "=" + headerString;  
			returnValue += headerSer;
		}
		if (this.payload != null) {
			var payloadString = encodeURIComponent(this.payload);
			var payloadSer = "&" + payloadName + "=" + payloadString;  
			returnValue += payloadSer;
		}
		return returnValue;
	}
	
	/**
	 * Deserializes a serialized message and initializes the objects parameters.
	 */
	this.deserialize=function(serializedMessage){
		var messageParts = serializedMessage.split("&");
		for(var i = 0; i < messageParts.length; i++){
			var nameValue = messageParts[i].split("=");
			switch(nameValue[0]){
			case typeName:
				this.type=nameValue[1];
				break;
			case topicName:
				this.topic=decodeURIComponent(nameValue[1]);
				break;
			case additionalHeaderName:
				var headerString = decodeURIComponent(nameValue[1]);
				this.additionalHeader = JSON.parse(headerString);
			case payloadName:
				this.payload=decodeURIComponent(nameValue[1]);
				break;
			}	
		}
	}	
}

// only use letters or numbers as characters

// PUBLISH message: additionalHeader is {f:"S"} or {f:"J"} representing that the payload is a string or JSON, 
// topic and payload are topic, payload of message
smash.SECommMessage.PUBLISH="pub"; 
// DISTRIBUTE message: additionalHeader is {f: string, s:[string, ...]} where f is defined as in the PUBLISH message, 
// and s representing subIds that should receive this message; topic and payload are as in PUBLISH message 
smash.SECommMessage.DISTRIBUTE="dis"; 
// SUSCRIBE message: additionalHeader is {subId: string}, payload==null, topic is subscription topic
smash.SECommMessage.SUBSCRIBE="sub"; 
// UNSUBSCRIBE message: additionalHeader is {subId: string}, topic==null, payload==null
smash.SECommMessage.UNSUBSCRIBE="uns";
// SUBCRIBE_ACK message: additionalHeader is {subId: string, isOk: boolean, err: string}, topic==null, payload == null
smash.SECommMessage.SUBSCRIBE_ACK="sac"; 

smash.SECommMessage.ERROR="err"; // TBD


//-----------------------------------------------------------------------------------------------
/**
 * Definitions of exceptions used by SECom
 */
smash.SECommErrors = {};
smash.SECommErrors.tunnelNotSetError = new Error ("The tunnel URI was not set. Please set the tunnel URI.");
smash.SECommErrors.componentNotFoundError = new Error ("The component could not be identified. Please declare the component correctly.");
smash.SECommErrors.securityTokenNotVerifiedError = new Error (smash.SecurityErrors.TOKEN_VERIFICATION_FAILED_MSG);
smash.SECommErrors.tunnelUnloadError = new Error (smash.SecurityErrors.TUNNEL_UNLOAD_MSG);
smash.SECommErrors.componentLoadError = new Error (smash.SecurityErrors.COMPONENT_LOAD_MSG);

/**
 * Links the SEHub and the SEHubClient together over the communication implemented by CommLib bridge
 *
 * TODO: Check if the component loading allows valid HTML.
 * TODO: Propagate the style of the enclosing tag into the iFrame
 * TODO: Check if there is a better way than polling to see if the tunnel's commLib has been registered
 */
smash.SEComm = function(){
	// The timer used to delay the phishing message. This makes sure that a page navigation does not cause phishing errors.
	// Setting it to 1 ms is enough for it not to be triggered on regular page navigations.
	var unloadTimer=1;
	// Variable storing the identifier for the setInterval if processing a registrationTimer	
	var registrationTimerProcess=null;
	// The URI of the component being manages by this SEComm.
	var componentURI=null;
	// The commLib of the tunnel
	var commLib=null;
	// Variable storing the identifier to clear when the setInterval is called
	var commLibPoll=null;
	// The HTML id of the component for which this is a SEComm
	var componentID=null;
	// A queue for outgoing messages. This queue is used when new send requests are done while we are still sending or receiving a message.
	var queueOut=[];
	// Variable storing the identifier for the setInterval if processing an output queue
	var queueOutProcess=null;
	// Variable storing a reference to the SEHub which is managing this SEComm
	var seHUB=null;
	// The iframe in which the component is loaded
	var myIframe = null;
	// The security token used for this component
	var securityTokenParent=null;
	// Variable storing the callback to the security listener function
	var securityListener=null;
	// This is used to make the object available to the private methods. This is a workaround for an error in the ECMAScript Language Specification which causes this to be set incorrectly for inner functions. See http://www.crockford.com/javascript/private.html
	var that=this;		
	// keeps track of the initialization
	var intialized=false;

	/**
	 * Sets the callback for security errors.
	 * 
	 * @param The callback for security errors.
	 */
	this.setSecurityListener=function(callback){
		securityListener=callback;
	}

	/**
	 * Prepares for loading a component into an iframe.
	 * @return The modified URI
	 */
	this.prepareForLoad=function(componentId, frameURI, seHub, loadTimeout) {
		if(debugEnabled)debug("Parent connecting to : " + componentId);
		// Store the SEHub
		seHUB=seHub;
		// Store the component Id
		componentID=componentId;
		// Check if the tunnel is set
		if (smash.SEComm.tunnelURI==null)throw smash.SECommErrors.tunnelNotSetError;
		// modify the URI
		securityTokenParent=smash._generateSecurityToken();
		// include the token twice since the child token value does not matter yet
		componentURI = frameURI + "?prov=http://providers.openajax.org/smash" + "&id=" + encodeURIComponent(componentId);
		var modifiedURIWithFragment = componentURI + "#100" + securityTokenParent + securityTokenParent + "000" + encodeURIComponent(componentId) + ":" + encodeURIComponent(smash.SEComm.tunnelURI);
		// Make the instance available for the tunnel.
		smash.SEComm.instances[componentId]=that;	
		// Set a timer which detects if the component loaded successfully
		// We are using an interval not to lose our evaluation context.
		registrationTimerProcess=setInterval(pollForIncomingCommLibTimeout,loadTimeout);

		return modifiedURIWithFragment;
	}

	function pollForIncomingCommLibTimeout(){
		clearInterval(registrationTimerProcess);
		registrationTimerProcess = null;
		//No CommLib has been registered.
		if(commLib==undefined) that.handleSecurityError(smash.SecurityErrors.COMPONENT_LOAD);
	}
	
	/**
	 * Gets the scope. Should only be used by the tunnel during INIT. 
	 * @return scope (object) the scope in which the callback needs to be called.
	 **/
	this.getScope=function(){
		return this;
	}

	/**
	 * Gets the callback. Should only be used by the tunnel during INIT. 
	 * @param c (string) the name of the callback method	 
	 **/
	this.getCallback=function(){
		return "messageReceived";
	}

	
	/**
	 * Called when the initialisaiton of the library is done and processes all messages in the queue
	 */
	this.initializationFinished=function(tunnelCommLib, token, clientURI, tunnelWindow){
		if(debugEnabled)debug("Tunnel commLib initialization finished. Processing outgoing queue. Security token: " + token);
		// verify the security token and clientURI
		if ((securityTokenParent!=token) || (clientURI!=componentURI)) {
			that.handleSecurityError(smash.SecurityErrors.TOKEN_VERIFICATION_FAILED);
			return false;
		}
		else {
			commLib=tunnelCommLib;		
			intialized=true;
			if(debugEnabled)debug("Token verified.");
			// register the onunload handler
			tunnelWindow.onunload=tunnelUnloadHandler;
			// switch the state to loaded in the seHUB. 
			seHUB.componentLoaded(componentID);
			// process the current outgoing queue.
			while (queueOut.length>0)commLib.send(queueOut.shift());
			return true;
		}
	}
	
	this.prepareForUnload = function() {
		// stop all timers
		if (registrationTimerProcess != null) {
			clearInterval(registrationTimerProcess);
			registrationTimerProcess = null;
		}
	}
	
	function securityListenerClosure(error, componentId) {
		return function() {
			securityListener(error, componentId);
		}
	}

	this.handleSecurityError=function (error){
		if (securityListener==null){
			throw new Error (error);							
		}
		else{
			securityListener(error,componentID);
		}
		return;
	}

	
	/** 
	 * 
	 */
	function tunnelUnloadHandler(){		
		if (securityListener==null){
			setTimeout("throw tunnelUnloadError;", unloadTimer);
		}
		else{
			setTimeout(securityListenerClosure(smash.SecurityErrors.TUNNEL_UNLOAD, componentID), unloadTimer);
		}						
	}
	/**
	 * Function processing the incomming data from commLib
	 *
	 * @param message The message containing the incomming data
	 */
	this.messageReceived=function (message){
		var msg=new smash.SECommMessage();
		msg.deserialize(message);
		switch(msg.type){
		case smash.SECommMessage.PUBLISH:
			if (msg.additionalHeader != null) {
				var payload = msg.payload;
				if (msg.additionalHeader.f == "J")
					payload = JSON.parse(msg.payload);
				seHUB.publishInternal(componentID, msg.topic, payload);
			} // else no additionalHeader defining the payload format. hence ignore the message
			break;
		case smash.SECommMessage.SUBSCRIBE:
			if (msg.additionalHeader != null)  {
				var isOk = seHUB.subscribeInternal(componentID, msg.additionalHeader.subId, msg.topic);
				var msgack = new smash.SECommMessage();
				msgack.type = smash.SECommMessage.SUBSCRIBE_ACK;
				msgack.additionalHeader={subId: msg.additionalHeader.subId, isOk: isOk, err: ""};
				send(msgack.serialize());
			}
			break;
		case smash.SECommMessage.UNSUBSCRIBE:
			if (msg.additionalHeader != null)
				seHUB.unsubscribeInternal(componentID, msg.additionalHeader.subId);
			break;
		}
	}
	/**
	 * Sends a published message to the partner component
	 */
	this.distribute=function(topic, matchingSubs, payload){
		var msg=new smash.SECommMessage();
		msg.type=smash.SECommMessage.DISTRIBUTE;
		msg.topic=topic;
		msg.additionalHeader = {s: matchingSubs};
		if ((typeof payload) == "string") {
			msg.additionalHeader.f = "S";
			msg.payload=payload;
		}
		else {
			msg.additionalHeader.f = "J";
			msg.payload = JSON.stringify(payload);
		}	
		send(msg.serialize());
	}

	function send(message) {
		// Queue the message if sending or if there is no communication partner yet
		if (intialized==false){
			queueOut.push(message);
		}
		else{
			commLib.send(message);
		}
	}
}

// Server relative URI for the tunnel on the server. For convenience, we initialize it with the location
// you will get when installing the library in the default locations.
smash.SEComm.servRelTunnelURI = "/hub11smashprovider/src/tunnel.html"; // TODO: eventually a better default.
// Static variable containing the URI of the tunnel file. 
// Make sure this is intialized and in the same domain as the main application. Most applications would never define
// this but modify SEComm.servRelTunnelURI or even leave the default.
smash.SEComm.tunnelURI= document.location.protocol + "//" + document.location.host + smash.SEComm.servRelTunnelURI;
// Static array which contains the list of the currently loaded instances. The array is indexed by the url of the child component. 
smash.SEComm.instances=[];

//-----------------------------------------------------------------------------------------------

/**
 * SEHubClient implementation linking the SEComm together with the component side logic.
 */
smash.SECommClient = function(clientName){
	// Storing the CommLib used for communicating
	var controllers=[];
	controllers["child"]=this;
	var commLib=new smash.CommLib(true, controllers, clientName);
	// This is used to make the object available to the private methods. This is a workaround for an error in the ECMAScript Language Specification which causes this to be set incorrectly for inner functions. See http://www.crockford.com/javascript/private.html
	var that=this;
	// A queue for outgoing messages. This queue is used when new send requests are done while we are still sending or receiving a message.
	var queueOut=[];
	// keeps track of the initialization
	var intialized=false;
	var securityListener=null;
	var jsonPayloadHeader = {f: "J"};
	var stringPayloadHeader = {f: "S"};
	var parsedTunnelUrl = null;	
	/**
	 * Publishes a message to a certain topic
	 * @param topic string
	 * @param data JSON|string
	 */
	this.publish=function(topic, data){
		var msg=new smash.SECommMessage();
		msg.type=smash.SECommMessage.PUBLISH;
		msg.topic=topic;
		if ((typeof data) == "string") {
			msg.additionalHeader = stringPayloadHeader;
			msg.payload=data;
		}
		else {
			msg.additionalHeader = jsonPayloadHeader;
			msg.payload = JSON.stringify(data);
		}	
		send(msg.serialize());
	}

	/**
	 * subscribes to a certain topic
	 */
	this.subscribe=function(subId, topic){
		var msg=new smash.SECommMessage();
		msg.type=smash.SECommMessage.SUBSCRIBE;
		msg.topic=topic;
		msg.additionalHeader = {subId: subId};
		send(msg.serialize());
	}

	/**
	 * Called when the initialisaiton of the library is done and processes all messages in the queue
	 */
	this.initializationFinished=function(tunnelUrl){
		if(debugEnabled)debug("Initialization finished. Processing outgoing queue.");
		parsedTunnelUrl = new ParsedUrl(tunnelUrl);

		intialized=true;	
		while (queueOut.length>0)commLib.send(queueOut.shift());
	}

	var _regex = new RegExp("^((http|https):)?(//([^/?#:]*))?(:([0-9]*))?([^?#]*)(\\?([^#]*))?");
	function ParsedUrl(url) {
		var matchedurl = url.match(_regex);
		this.scheme = (matchedurl[2] == "") ? null : matchedurl[2];
		this.host = (matchedurl[4] == "") ? null : matchedurl[4];
		this.port = (matchedurl[6] == "") ? null : matchedurl[6];
		this.path = (matchedurl[7] == "") ? null : matchedurl[7];
		this.query = (matchedurl[8] == "") ? null : matchedurl[8];
	}

	/**
	 * unsubscribes
	 */
	this.unsubscribe=function(subId){
		var msg=new smash.SECommMessage();
		msg.type=smash.SECommMessage.UNSUBSCRIBE;
		msg.additionalHeader={subId: subId};
		send(msg.serialize());
	}
	
	function send(message) {
		// Queue the message if sending or if there is no communication partner yet
		if (intialized==false){
			queueOut.push(message);
		}
		else{
			commLib.send(message);
		}
	}


	/**
	 * Function processing the incomming data from commLib
	 *
	 * @param message The message containing the incomming data
	 */
	this.messageReceived=function (message){
		var msg=new smash.SECommMessage();
		msg.deserialize(message);
		// parse the JSON payload
		if (msg.type == smash.SECommMessage.DISTRIBUTE) {
			var header = msg.additionalHeader;
			if ((header != null) && (header.f == "J"))
				msg.payload = JSON.parse(msg.payload);
		} 
		if ((msg.type == smash.SECommMessage.DISTRIBUTE) || (msg.type == smash.SECommMessage.SUBSCRIBE_ACK))
			that.handleIncomingMessage(msg);
	}

	
	this.handleSecurityError=function (error){
		if (securityListener==null){
			throw new Error (error);							
		}
		else{
			securityListener(error,componentID);
		}
		return;
	}


	/**
	 * Sets the callback for security errors.
	 * 
	 * @param The callback for security errors.
	 */
	this.setSecurityListener=function(callback){
		securityListener=callback;
	}

	/**
	 * This method is the location for the callback to the SECommClient library.
	 * The application using this library overrides this method with its own implementation.
	 * HACK: this is terrible from a layering perspective. Ideally all message formatting details, such
	 * as header formats should be handled at this layer alone.
	 * The default behavior is to alert a message.
	 *
	 * @param message The actual message.
	 */
	this.handleIncomingMessage=function(message){
		alert("SECommClient\n\nTopic: " + message.topic + "\n\nPayload: " + message.payload);
	}
}

/**
 * Provides the low level communication layer.
 * @param child (boolean) indicating if this is a child iframe or not.  
 * @param controllers (object []) an array indexed by the clientName of objects implementing the controller interface.
 * @param clientName - only explicitly passed for the child iframe
 * 
 * controller.messageReceived - called when the commlib recieves an incomming message.
 * controller.initializationFinished - called when the commlib finished its initialzation.
 * controller.handleSecurityError - called when a security error occurs.
 * 
 */
smash.CommLib=function(child, controllers, clientName){
	/**BEGIN of communcation protocol **/		
	/*
	  Message format:
	 | Message Type | Message Sequence Number | Security Token Parent | Security Token Child | ACK              | ACK Message Sequence Number      | Payload             |
	 | 1 character      | 2 characters        |  x characters         | x characters         | 1 character  | 2 characters                         | varable length  | 
	*/
	// Init message payload=communication partner url
	var INIT="1";		
	// An ack message without any payload. The reciever is not supposed to ack this message therefore the message sequence number will be 00.
	var ACK="2";		
	// The part message indicates that this is a message that needed to be split up. It will contain the payload of a part of the total message.
	var PART="3";
	// The end message indicates that this is the last part of a split up message. The full message has arrived after processing this message.
	var END="4";		
	
	/** END of communcation protocol **/
	// This is used to make the object available to the private methods. This is a workaround for an error in the ECMAScript Language Specification which causes this to be set incorrectly for inner functions. See http://www.crockford.com/javascript/private.html
	var that=this;		
	// polling and queue processing interval
	var interval=100;
	// The maximul length of a URL. If the message is longer it will be split into different parts.
	var urlLimit = 4000;
	// Protocol overhead excluding security token overhead
	var protocolOverhead=6;
	// Need to do an acknowledgement
	var ack=0;
	// Raw incoming data
	var currentHash=null;
	// The newly decoded incoming message
	var messageIn=null;
	// The last decoded incoming message
	var previousIn=null;
	// The currently transmitted message
	var messageOut=null;
	// The previously transmitted message
	var previousOut=null;		
	// The url of the  partner
	var partnerURL=null;
	// The window object of the partner
	var partnerWindow=null;
	// A queue for outgoing messages. This queue is used when new send requests are done while we are still sending or recieving a message.
	var queueOut=[];
	// Storing the last sent message number
	var msn=00;
	// Buffer for storing the incoming message parts
	var messageBuffer="";
	// Variable storing the timerId of the message timer.
	var timerId=null;
	// Two security tokens - One created by the parent frame (the manager) and one by the child frame (the client)
	var securityTokenParent=null;
	var securityTokenChild=null;
	// 
	
	/**
	 * Sends a message to the communication partner
	 * @param message (string) the message that needs to be delivered to the communication partner
	 */
	this.send=function(message){
		// check if we are properly initialized
		if (partnerURL==null){
			if(debugEnabled)debug("Trying to send without proper initialization. Message will be discarded. " +  message);
			return;
		}
		if(debugEnabled)debug("Sending: " + message);
		// URL encode the message
		// var encodedMessage=encodeURIComponent(message);
		var encodedMessage=message;
		// determine the payload size
		var payloadLength=urlLimit-protocolOverhead-smash._securityTokenOverhead-partnerURL.length;
		// DEBUG LARGE MESSAGES 
		//if(debugEnabled)payloadLength=1;
		// Split up into separate messages if necessary
		var currentMessage=encodedMessage;
		while (currentMessage.length>0){
			// split up and put in output queue
			var part=currentMessage.substr(0,payloadLength);
			currentMessage=currentMessage.substr(payloadLength);
			if (currentMessage==0){
				queueOut.push({type: END, payload: part});
			}
			else{
				queueOut.push({type: PART, payload: part});
			}
		}
	}
	
	/**
	 * The timer triggering the flow of messages through the system.
	 */
	function messageTimer(){
		// check if there is a new message
		if(checkMessage()){
			// check if it can be decoded properly
			if (decodeMessage()){
				// check if it is conform the security requirements
				if (checkSecurity()){
					// process it
					processMessage();
				}					
			}				
		}
		// Only sent if an ack was received for the last transmitted message.
		if (checkAck()){
			// send anything that might be in the out queue
			sendMessage();
		}
	}
	
	/**
	 * Returns true if the previously transmitted message was acknowledged.
	 * 
	 * Possible exception situations to take into account: 
	 * - One of the parties takes two turns in a row.
	 *   p   p   c   
	 * c p1  - 
	 * p         p1'
	 * 
	 *   c   p   p   c
	 * c     ac1 p1  
	 * p c1          p1'
 	 * 
	 */
	function checkAck(){
		// No ack is expected for an ack.
		if (previousOut.type==ACK)return true;
		// Ack is received. 
		if ((previousOut.msn==messageIn.ackMsn) && (messageIn.ack==1)) return true;
		// Wait for the ack to arrive.
		if (debugEnabled)debug("Waiting for ACK : " + previousOut.msn)
		return false;
	}
	
	/**
	 * Helper method providing a new  message sequence number
	 * @return (string) the new sequence number
	 */
	function getNewMsn(){
		msn++;
		if (msn==100) msn=0;
		if (msn<10) return "0" + msn;
		return "" + msn;			
	}
	
	/**
	 * Checks the information after the hash to see if there is a new incomming message.
	 */
	function checkMessage(){
		//Can't use location.hash because at least Firefox does a decodeURIComponent on it.
		var urlParts = window.location.href.split("#");
		if(urlParts.length == 2){
			var newHash = urlParts[1];
			if(newHash!="" && newHash != currentHash){
				currentHash = newHash;
				return true;
			}
		}
		return false;
	}

	/**
	 * Decodes an incomming message and checks to see if it is syntactially valid.
	 */		
	function decodeMessage(){						
		var type=currentHash.substr(0,1);
		var msn=currentHash.substr(1,2);
		var nextStart = 3;
		var tokenParent=currentHash.substr(nextStart,smash._securityTokenLength);
		nextStart += smash._securityTokenLength;
		var tokenChild=currentHash.substr(nextStart,smash._securityTokenLength);
		nextStart += smash._securityTokenLength;
		var ack=currentHash.substr(nextStart,1);
		nextStart += 1;
		var ackMsn=currentHash.substr(nextStart,2);
		nextStart += 2;
		// The payload needs to stay raw since the uri decoding needs to happen on the concatenated data in case of a large message
		var payload=currentHash.substr(nextStart);
		if(debugEnabled)debug("In : Type: " + type + " msn: " + msn + " tokenParent: " + tokenParent + " tokenChild: " + tokenChild + " ack: " + ack + " msn: " + ackMsn + " payload: " + payload);			
		messageIn={type: type, msn: msn, tokenParent: tokenParent, tokenChild: tokenChild, ack: ack, ackMsn: ackMsn, payload: payload}
		return true;
	}

	/**
	 * Check if there have been any security breaches in the message.
	 */				
	function checkSecurity(){			
		// Check the security tokens
		if (messageIn.type!=INIT && (messageIn.tokenParent!=securityTokenParent || messageIn.tokenChild!=securityTokenChild)){
			if(debugEnabled)debug("Security token error: Invalid security token received. The message will be discarded.");
			handleSecurityError(smash.SecurityErrors.INVALID_TOKEN);
			return false;
		}		
		// Attacks should never pass the security check. Code below is to debug the implementation.
		if(debugEnabled){
			if (messageIn.type!=INIT && messageIn.type!=ACK && messageIn.type!=PART && messageIn.type!=END){
				if(debugEnabled)debug("Syntax error: Message Type. The message will be discarded.");
				return false;
			}
			if (!(messageIn.msn>=0 && messageIn.msn<=99)){
				if(debugEnabled)debug("Syntax error: Message Sequence Number. The message will be discarded.");
				return false;
			}
			if (!(messageIn.ack==0 || messageIn.ack==1)){
				if(debugEnabled)debug("Syntax error: ACK. The message will be discarded.");
				return false;
			}
			if (!(messageIn.ackMsn>=0 && messageIn.ackMsn<=99)){
				if(debugEnabled)debug("Syntax error: ACK Message Sequence Number. The message will be discarded.");
				return false;
			}
		}
		return true;
	}

	/**
	 * Process the incoming message.
	 */						
	function processMessage(){
		ack=1;
		// The child is initialized as soon as there is an ack for the init message sent by the child.
		if (messageIn.type!=INIT && child && previousOut.type==INIT && messageIn.ack=="1" && previousOut.msn==messageIn.ackMsn)controller.initializationFinished(partnerURL);	
							
		// Call the actual processing functions
		switch(messageIn.type){
			case INIT:
				processInit();
				break;
			case ACK:
				processAck();
				break;
			case PART:
				processPart();
				break;
			case END:
				processEnd();
				break;					
		}
		// Set the processed message as the previousIn message
		previousIn=messageIn;		
	}

	/**
	 * Implementation of the INIT message type
	**/
	function processInit(){
		var parts = messageIn.payload.split(":");
		var cname = decodeURIComponent(parts[0]);
		partnerURL=decodeURIComponent(parts[1]);
		securityTokenParent=messageIn.tokenParent;
		securityTokenChild=messageIn.tokenChild;
		// Initialize a component
		if (child){
			if (clientName != null) cname = clientName; // override what is read from the URL
			// generate a real security token for the child
			securityTokenChild = smash._generateSecurityToken();
			// GUI which will be used to name the iFrame tunnel.
			var tunnelGUID="3827816c-f3b1-11db-8314-0800200c9a66"
			// Generate the hidden iframe for communicating
			var iframe = document.createElement("iframe");

			var currentClientURI = encodeURIComponent(window.location.href.split("#")[0]);
			var initialClientURI = currentClientURI;
			if (smash._initialClientURI) {
				initialClientURI = encodeURIComponent(smash._initialClientURI);
			}
			var initpayload = encodeURIComponent(cname) + ":" + currentClientURI + ":" + initialClientURI;

			// sending an ack for msn "00" to the tunnel, since have processed the INIT message,
			// and so that the INIT message the component is sending to the tunnel will result
			// in an ack to be sent back.
			iframe.src=partnerURL + "#100" + securityTokenParent + securityTokenChild + "100" + initpayload;
			iframe.name=tunnelGUID;
			iframe.id=tunnelGUID;
			document.body.appendChild(iframe);
			if (navigator.userAgent.indexOf("Safari") == -1) {
				iframe.style.position = "absolute";
			}
			iframe.style.left = iframe.style.top = "0px";
			iframe.style.height = iframe.style.width = "1px";
			iframe.style.visibility = "hidden";
			// We do not send an ack directly to the parent frame since it is impossible to directly communicate with it in IE7
			// The ack is done indirectly when the registerTunnelCommLib is done
			ack=0;
			// set up the partner window
			partnerWindow=window.frames[tunnelGUID];
			// store the last sent message - will be used to detect intialization and for detecting security breaches
			previousOut={type: INIT, msn: "00", tokenParent: securityTokenParent, tokenChild: securityTokenChild, ack: "0", ackMsn: "00", payload: initpayload}; // only using type and msn of previousOut. presumably the rest is for FDK's retransmit stuff? should get rid of this complexity
			// set the controller for this component
			controller=controllers["child"];
		}
		// Initialize a tunnel
		else{
			var initialClientURI = decodeURIComponent(parts[2]);
			// set up the partner window
			partnerWindow=window.parent;
			// set the controller for this component
			controller=controllers[cname];
			var success = controller.initializationFinished(that, securityTokenParent, initialClientURI, window);
			if (!success) ack = 0; // don't send an ack signalling the completion of connection setup.
			// store the last sent message - will be used to detect intialization and for detecting security breaches
			previousOut={type: INIT, msn: "00", tokenParent: securityTokenParent, tokenChild: securityTokenChild, ack: "0", ackMsn: "00", payload: (encodeURIComponent(cname) + ":" + encodeURIComponent(window.location.href.split("#")[0]))}; // only using type and msn of previousOut. presumably the rest is for FDK's retransmit stuff? should get rid of this complexity				
		}
		if (partnerWindow==null) {
			if(debugEnabled)debug("Init failed.");
		}
	}	
			
	/**
	 * Implementation of the ACK message type
	**/
	function processAck(){
		// do not ack an ack
		ack=0;
	}

	/**
	 * Implementation of the PART message type
	**/
	function processPart(){
		// Process message
		messageBuffer+=messageIn.payload;
	}		
	
	/**
	 * Implementation the END message type
	**/
	function processEnd(){
		// Process message
		messageBuffer+=messageIn.payload;
		// messageBuffer=decodeURIComponent(messageBuffer);
		if(debugEnabled)debug("Received: " + messageBuffer);
		controller.messageReceived(messageBuffer);
		messageBuffer="";
	}
	
	/**
	 * Send a reply to the incoming message.
	 */								
	function sendMessage(){						
		// If there is nothing in the queue and an ack needs to be sent put the ack on the queue;
		if (queueOut.length==0 && ack==1){
			// The correct values will be filled in later. Just push a clean ack message
			queueOut.push({type: ACK, payload: ""});
		}
		// Process the output queue
		if (queueOut.length!=0){
			messageOut=queueOut.shift();
			// Fill in the security token
			messageOut.tokenParent=securityTokenParent;
			messageOut.tokenChild=securityTokenChild;
			// Get a new sequence number
			messageOut.msn=getNewMsn();
			// Fill in the right ack values 
			// The protocol keeps acking the last received message to ensure that there are no 
			// problems with overwriting a pure ack message. Which could happen because there is 
			// no waiting for an ack of an ack.			
			messageOut.ack="1";
			messageOut.ackMsn=previousIn.msn;
			// turn of the ack
			ack=0;			
			writeToPartnerWindow();
		}
	}				
	
	/**
	 * Writes the message to the partner window's fragment id
	**/		
	function writeToPartnerWindow(){			
		var url = partnerURL + "#" + messageOut.type + messageOut.msn + messageOut.tokenParent + messageOut.tokenChild + messageOut.ack + messageOut.ackMsn + messageOut.payload;
		//Safari won't let us replace across domains.
		if(navigator.userAgent.indexOf("Safari") == -1){
			partnerWindow.location.replace(url);
		}else{
			partnerWindow.location = url;
		}				

		previousOut=messageOut;
		if(debugEnabled)debug("Out: Type: " + messageOut.type + " msn: " + messageOut.msn + " tokenParent: " + messageOut.tokenParent + " tokenChild: " + messageOut.tokenChild + " ack: " + messageOut.ack + " msn: " + messageOut.ackMsn + " payload: " + messageOut.payload);
	}		
	
	/**
	 * Default handler of the security listener. If a security error occurs, the CommLib is switched off. And communication is no longer possible.
	 * 
	 */
	function handleSecurityError(error){
		// Stop the communication
		clearInterval(timerId);	
		// If there	is a securityListener inform the controller of what happened.
		controller.handleSecurityError(error);
	}
		
	// Start listening for incoming messages
	timerId=setInterval(messageTimer, interval);
};


//-----------------------------------------------------------------------------------------------
// now register oneself
(function() {
	if (typeof this["OpenAjax"] != "undefined") {
		if (typeof this["OpenAjax"]["hubspi"] != "undefined") {
			smash._oahubspi = OpenAjax.hubspi;
			var suceess = smash._oahubspi.register(smash);
		}
    	}
})();

//-----------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------
